local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL:AddCallbackHandler(Global)
GGL.Options.Sets = Global

local L = GGL.Locale

-- lua
local pairs, ipairs = pairs, ipairs
local tblinsert = table.insert

-- WoW
local NUM_LE_ITEM_CLASSS = _G.NUM_LE_ITEM_CLASSS

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local Data = {}
local ADD_SET_STRING = format("|A:%s:14:14:0:0|a |cff00ff00%s|r", GGL.ATLAS_TEXTURE["+"], L["Add new"])
local CONFIG_STRING = format("|A:%s:16:16:0:0|a %s", GGL.ATLAS_TEXTURE["config"], L["Config"])

local TAB_GROUP_SETTINGS_TAB = { text = CONFIG_STRING, value = "__config" }

-- ##################
-- local helper
-- ##################

local function PerformeSaveDB()
    if Data["selectedDB"] then
        Data["selectedSet"].setting = GGL:Serialize(Data["selectedDB"])
    end
end

local function DropdownGroup_CreateContent()
    local ddGroup = Data["ddGroup"]
    if not ddGroup then return end
    local setID = Data["setID"]
    local db = GGL.db.Config.Sets[setID]
    local groupID = GGL.db.Config.Sets.LastSelected[setID]
    groupID = GGL.db.Config.Sets[setID][groupID] and groupID or nil
    -- fill list
    local ddData, ddSort = {}, {}
    for k, v in pairs(db) do
        if not groupID then groupID = k end
        ddData[k] = v.name
        ddSort[ #ddSort+1 ] = k
    end
    -- last line should be Add New
    ddData["__add"] = ADD_SET_STRING
    ddSort[ #ddSort+1 ] = "__add"
    -- set grouplist
    ddGroup:SetGroupList(ddData, ddSort)

    -- select current or first line
    ddGroup:SetGroup(groupID)
end

local function DropdownGroup_OnGroupSelected(self, event, groupID)
    local setID = Data["setID"]
    local db = GGL.db.Config.Sets
    if groupID == "__add" then
        -- add a new one
        local newGroupID = GGL.GetTime()
        db[setID][newGroupID] = {
            name = L["New"],
            setting = GGL:Serialize({})
        }
        db.LastSelected[setID] = newGroupID
        Data["selectedSetID"] = newGroupID

        DropdownGroup_CreateContent()
    else
        db.LastSelected[setID] = groupID
        Data["selectedSetID"] = groupID
        Data["selectedSet"] = db[setID][groupID]
        local retC, sDb = GGL:Deserialize( db[setID][groupID].setting)
        Data["selectedDB"] = sDb
        if Data["tabGroup"] then
            Data["tabGroup"]:SelectTab(Data["selectedTabID"] or Data["firstTabID"])
        end
    end

end

local function DeleteSetFromDB(setID, groupID)
    if GGL.db.Config.Sets[setID][groupID].protected then return end
    GGL.db.Config.Sets[setID][groupID] = nil
    DropdownGroup_CreateContent()
end

local function DeleteSetFromDB_ShowDeleteConfirm(self, event)
    GGL.Options:OpenConfirm(
        format(L["Delete set '|cffff0000%s|r'?"], Data["selectedSet"].name),
        DeleteSetFromDB,
        nil,
        Data["setID"], Data["selectedSetID"]
    )
end

local function NameChange_OnEnterPressed(self, event, text)
    GGL.db.Config.Sets[ Data["setID"] ][ Data["selectedSetID"] ].name = text
    DropdownGroup_CreateContent()
    self:ClearFocus()
end

local function TabGroup_OnGroupSelected(self, event, tabID)
    local content = Data["scroll"]
    local method
    -- clear old things
    content:ReleaseChildren()

    Data["selectedTabID"] = tabID

    if tabID == "__config" then
        local base = AceGUI:Create("SimpleGroup")
        base:SetFullWidth(true)
        base:SetLayout("Flow")
        content:AddChild(base)

        local name = AceGUI:Create("EditBox")
        name:SetLabel(L["Name"])
        name:SetText(Data["selectedSet"].name)
        name:SetRelativeWidth(0.5)
        name:SetCallback("OnEnterPressed", NameChange_OnEnterPressed)
        base:AddChild(name)
        -- build the delete button
        if not Data["selectedSet"].protected then
            local delete = AceGUI:Create("Button")
            delete:SetAutoWidth(true)
            delete:SetText(DELETE)
            delete:SetCallback("OnClick", DeleteSetFromDB_ShowDeleteConfirm)
            delete:SetAutoWidth(true)
            base:AddChild(delete)
        end

        local contentNew = AceGUI:Create("SimpleGroup")
        contentNew:SetFullWidth(true)
        content:AddChild(contentNew)

        content = contentNew
        method = self:GetUserData("ConfigSelectedMethod")
    else
        method = self:GetUserData("SelectedMethod")
    end

    if method then
        method(content, Data["selectedDB"], tabID)
    end
end


local Helper = {
    ["Data"] = Data,
    ["SetTabGroupContent"] = function(self, content, addSettingsTab)
        local tabGroup = self.Data["tabGroup"]
        if not tabGroup then return end
        content = content or {}
        if addSettingsTab then
            tblinsert(content, 1, TAB_GROUP_SETTINGS_TAB)
        end
        self.Data["selectedTabID"] = nil
        self.Data["firstTabID"] = content[1].value
        tabGroup:SetTabs(content)
        tabGroup:SelectTab(self.Data["firstTabID"])
    end,
    -- method(content, tabID)
    ["SetOnTabGroupSelected"] = function(self, method)
        self.Data["tabGroup"]:SetUserData("SelectedMethod", method)
    end,
    -- method(content, tabID)
    ["SetOnTabGroupConfigSelected"] = function(self, method)
        self.Data["tabGroup"]:SetUserData("ConfigSelectedMethod", method)
    end,
    ["RefreshSelectedTab"] = function(self)
        self.Data["tabGroup"]:SelectTab(Data["selectedTabID"] or Data["firstTabID"])
    end,
    ["SaveDB"] = function(self)
        PerformeSaveDB()
    end,
    ["UpdateDropdownText"] = function(self)
        DropdownGroup_CreateContent()
    end,
}
-- ##################
-- Global
-- ##################
function Global:GetData()
    return Data
end

function Global:GetBaseGroup(container, setID)
    local ddGroup = AceGUI:Create("DropdownGroup")
    ddGroup:SetCallback("OnGroupSelected", DropdownGroup_OnGroupSelected)
    ddGroup:SetLayout("Fill")

    local tabGroup = AceGUI:Create("TabGroup")
    tabGroup:SetCallback("OnGroupSelected", TabGroup_OnGroupSelected)
    tabGroup:SetLayout("Fill")

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")

    -- add container+
    container:AddChild(ddGroup)
    ddGroup:AddChild(tabGroup)
    tabGroup:AddChild(scroll)

    Data["ddGroup"] = ddGroup
    Data["tabGroup"] = tabGroup
    Data["scroll"] = scroll
    Data["setID"] = setID

    Helper:SetOnTabGroupSelected(nil)
    Helper:SetOnTabGroupConfigSelected(nil)

    -- init
    DropdownGroup_CreateContent()

    return Helper
end

-- ##################
-- options
-- ##################
local SET_LIST_SORTED = {
    --{ GGL.SET_ID_LIST.Set,          L["Sets"] },
    { GGL.SET_ID_LIST.PriceSource,  L["Price Source"],  canDisabled = false },
    { GGL.SET_ID_LIST.Item,         L["Items"],         canDisabled = false },
    { GGL.SET_ID_LIST.Zone,         L["Zone"],          canDisabled = true },
    { GGL.SET_ID_LIST.Instance,     L["Instance"],      canDisabled = true },
    { GGL.SET_ID_LIST.LootSource,   L["Loot Source"],   canDisabled = true },
}

local function GetSetDropDownList(setID)
    local db = GGL.db.Config.Sets[setID]
    local list = {}

    for setKey, setData in pairs(db) do
        list[setKey] = setData.name
    end

    return list
end

local function SetDropDown_OnValueChanged(self, event, setKey)
    local db = self:GetUserData("db")
    local setID = self:GetUserData("setID")

    local refresh = db[setID] == true and true or false

    if setKey == "Base" then
        db[setID] = nil
    else
        db[setID] = setKey
    end

    Helper:SaveDB()

    if refresh then
        Helper:RefreshSelectedTab()
    end
end

local function DisableCheck_OnValueChanged(self, event, state)
    local db = self:GetUserData("db")
    local setID = self:GetUserData("setID")

    if state then
        db[setID] = nil
    else
        db[setID] = true
    end

    Helper:SaveDB()

    Helper:RefreshSelectedTab()
end


local function SetOnTabGroupConfigSelected(content, db, tabID)
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")

    for num, set in ipairs(SET_LIST_SORTED) do

        local line = AceGUI:Create("Heading")
        line:SetFullWidth(true)
        base:AddChild(line)

        local check
        if set.canDisabled then
            check = AceGUI:Create("CheckBox")
            if db[set[1]] == true then
                check:SetValue(false)
            else
                check:SetValue(true)
            end
            check:SetCallback("OnValueChanged", DisableCheck_OnValueChanged)
            check:SetUserData("setID", set[1])
            check:SetUserData("db", db)
            check:SetWidth(30)
            check:SetLabel("")
            GGL.GUI:AddToolTipForAceGUIWidget(check, set[2], L["Should only be enabled if needed."])
            base:AddChild(check)
        end

        local text = AceGUI:Create("Label")
        if check and check:GetValue() == false then
            text:SetText(set[2].." - "..L["|cffff0000Disabled|r"])
        else
            text:SetText(set[2])
        end
        base:AddChild(text)

        if not set.canDisabled or (check and check:GetValue() == true) then
            local dropdown = AceGUI:Create("Dropdown")
            dropdown:SetList(GetSetDropDownList(set[1]))
            dropdown:SetValue(db[set[1]] or "Base")
            dropdown:SetCallback("OnValueChanged", SetDropDown_OnValueChanged)
            dropdown:SetUserData("setID", set[1])
            dropdown:SetUserData("db", db)
            base:AddChild(dropdown)
        end
    end

    content:AddChild(base)
end

local function Options(AceGUI, container)
    local SetsHelper = Global:GetBaseGroup(container, GGL.SET_ID_LIST.Set)

    -- empty for now
    local tabs = {}

    --Helper:SetOnTabGroupSelected(SetOnTabGroupSelected)
    Helper:SetOnTabGroupConfigSelected(SetOnTabGroupConfigSelected)

    Helper:SetTabGroupContent(tabs, true)
end

GGL.Options:AddIntoTree(Options, L["Loot Filter"], "sets")