-- ##################
--- EVENTS ( GGL.Templates.WoWPlayer:RegisterCallbackEvent )
-- "NewPlayerRegistered" (playerTemplate)
-- "PlayerUpdate" (playerTemplate)
-- "DisplayNameUpdate" (playerTemplate, displayName)
-- ##################
local AddonName, GGL = ...
local Templates = GGL.Templates

local WoWPlayer, WoWPlayerHelper = Templates:Register("WoWPlayer")

-- lua
local assert, type = assert, type
local pairs = pairs
local match = string.match

-- wow
local GetNormalizedRealmName = GetNormalizedRealmName


-- ##################
-- local
-- ##################
local PlayerRegister = {}

local NAME_MATCH = "^(.*)%-(.*)"

-- ##################
-- static
-- ##################
local Static = {}
Templates.WoWPlayer = Static

function Static:Get(playerName)
    return playerName and PlayerRegister[playerName] or nil
end

function Static:GetPlayer()
    if not self.player then
        local name = UnitName("player")
        self.player = Templates:Get("WoWPlayer", name)
    end
    return self.player
end

function Static:GetPlayerSender()
    if not self.playerSender then
        self.playerSender = format("%s-%s", UnitName("player"), GetNormalizedRealmName())
    end
    return self.playerSender
end

-- method(WoWPlayer)
function Static:RegisterCallbackEvent(event, method)
    WoWPlayerHelper:RegisterCallbackEvent(event, method)
end

function Static:UnregisterCallbackEvent(event, method)
    WoWPlayerHelper:UnregisterCallbackEvent(event, method)
end


-- ##################
-- template
-- ##################
function WoWPlayer:Constructor(playerName)
    assert(playerName, "No 'playerName' found ("..(playerName or "nil")..")")
    playerName = strtrim(playerName)
    if strtrim(playerName) == "" then return end

    if PlayerRegister[playerName] then
        return PlayerRegister[playerName]
    end

    local name, server = match(playerName, NAME_MATCH)
    self.characterName = name or playerName
    self.realmName = server or GetNormalizedRealmName()
    self.fullName = self.characterName.."-"..self.realmName
    self.syncID = self.fullName
    self.displayName = self.fullName
    self.sender = self.fullName
    self.isOnline = true
    self.isPlayer = (UnitName("player") == self.characterName)
    self.factionName = UnitFactionGroup("player")
    self.timeOffset = 0

    -- players from same realm can get called only with name also...
    if self.realmName == GetNormalizedRealmName() then
        PlayerRegister[self.characterName] = self
    end
    PlayerRegister[self.fullName] = self

    if not GGL.db.PlayerRegister[self.syncID] then
        GGL.db.PlayerRegister[self.syncID] = {}
    end

    GGL:AddCallbackHandler(self)

    self:Update()

    WoWPlayerHelper:FireCallbackEvent("NewPlayerRegistered", self)
end

function WoWPlayer:Update()
    if not GGL.db.PlayerRegister[self.syncID].displayName then
        GGL.db.PlayerRegister[self.syncID].displayName = self.fullName
    end
    self.displayName = GGL.db.PlayerRegister[self.syncID].displayName or self.fullName

    self:FireCallbackEvent("PlayerUpdate", self)
    WoWPlayerHelper:FireCallbackEvent("PlayerUpdate", self)
end

function WoWPlayer:SetDisplayName(displayName)
    displayName = displayName and GGL.FormatDisplayName(displayName) or nil
    GGL.db.PlayerRegister[self.syncID].displayName = displayName == "" and nil or displayName
    self:Update()
    self:FireCallbackEvent("DisplayNameUpdate", self, displayName)
    WoWPlayerHelper:FireCallbackEvent("DisplayNameUpdate", self, displayName)
end

function WoWPlayer:GetDisplayName()
    return GGL.db.PlayerRegister[self.syncID].displayName
end

function WoWPlayer:SetGGLData(version)
    self.gglVersion = version
end

function WoWPlayer:CanSendMessage(ignoreGGL)
    if not ignoreGGL and not self.gglVersion then
        return false
    elseif self.isOnline then
        return true
    end
end

function WoWPlayer:SetOnline(state)
    --self.isOnline = state
end

function WoWPlayer:SetTimeOffset(offset)
    self.timeOffset = offset or 0
end

function WoWPlayer:GetTimeOffset()
    return self.timeOffset
end

-- Events
-- ERR_CHAT_PLAYER_NOT_FOUND_S:gsub("%%s", "(.+)");
local ERR_CHAT_PLAYER_NOT_FOUND_S = ERR_CHAT_PLAYER_NOT_FOUND_S:gsub("%%s", "(.+)")

local EventFrame = CreateFrame("frame")

local function OnEvent(self, event, text)
    if text then
        local name = match(text, ERR_CHAT_PLAYER_NOT_FOUND_S)
        if name and PlayerRegister[name] then
            PlayerRegister[name]:SetOnline(false)
        end
    end
end

EventFrame:SetScript("OnEvent", OnEvent)
EventFrame:RegisterEvent("CHAT_MSG_SYSTEM")